# -*- coding: utf-8 -*-

from m3.actions import ControllerCache
from m3_ext.ui import all_components as ext
from m3_ext.ui.controls.buttons import ExtButton
from m3_ext.ui.containers.grids import ExtGridColumn
from m3_ext.ui.fields.complex import ExtDictSelectField
from m3_ext.ui.icons import Icons

from kinder.core import ui_helpers
from kinder.core.declaration_status.models import DeclarationStatusSewn as DSS

from lipetsk_specifics.models import (
    PrivilegeConfirmationAttributes,
    PrivilegeOwnerEnum as POE)
from lipetsk_specifics.models import Department

from lipetsk_specifics.constants import (
    ReturnReasonEnum,
    NOTIFICATION_MODE,
    PRIVILEGE_CONFIRMATION_STATUSES_MAP as PCS)


class PupilTransferListWindowExtension(object):
    """
    Расширяем окно списка переводов детей
    """
    @classmethod
    def build_extension(cls, list_win):
        pack = ControllerCache.find_pack(
            'lipetsk_specifics.actions.PupilTransferPrintPack')
        list_win.print_url = pack.print_action.get_absolute_url()
        list_win.template_globals = 'ui-js/pupil-transfer-list-print-window.js'
        list_win.items[0].top_bar.items.insert(
            5,
            ExtButton(
                text=u'Распечатать заявление',
                icon_cls=Icons.PRINTER,
                handler='print_transfer_info'
            )
        )
        return list_win


class DeclarationEditWindowExtension(object):
    """
    Расширяем окно редактирования заявки
    Добавляем в грид поле статус подтверждения льготы
    """

    @classmethod
    def build_extension(cls, edit_win):
        privilege_confirmation_status = ExtGridColumn(
                data_index='status', header=u"Статус")

        edit_win.unit_privilege_tab.priv_grid.columns.append(
            privilege_confirmation_status)

        return edit_win


class DeclarationPrivilegeAddEditWindowExtension(object):
    """
    Расширяем окно редактирования льготы
    Добавляем принадлежность льготы
    Добавляем владельца льготы
    Добавляем статус подтверждения льготы
    Добавляем кнопку подтверждения льготы
    Убираем поля документа
    """

    @classmethod
    def build_extension(cls, add_edit_win, action, context):
        win = add_edit_win

        win.NEED_DELEGATE = list(POE.NEED_DELEGATE)

        win.template_globals = 'ui-js/privilege-edit-window.js'

        win.PCS_YES = PCS[True]
        win.PCS_NO = PCS[False]

        pack = ControllerCache.find_pack(
            'lipetsk_specifics.actions.'
            'PrivilegeConfirmationAttributesPack')

        win.form.items.remove(win.field__doc_type_id)
        win.form.items.remove(win.field__doc_series)
        win.form.items.remove(win.field__doc_number)

        win.privilege_owner = ui_helpers.make_combo_box(
            anchor='100%',
            allow_blank=True,
            data=list(POE.get_choices()),
            label=u"Принадлежность льготы",
            name='privilege_owner')

        win.delegate = ext.ExtDictSelectField(
            region='north',
            anchor='100%',
            label=u"Обладатель льготы",
            name='delegate_id',
            display_field='fullname',
            ask_before_deleting=False,
            hide_edit_trigger=True,
            hide_dict_select_trigger=True,
            hide_trigger=False,
            allow_blank=True)

        win.delegate.pack = ControllerCache.find_pack(
            'lipetsk_specifics.actions.ChildrenDelegatePack').__class__
        win.delegate.editable = False

        win.privilege_confirmation_status = ext.ExtStringField(
            label=u"Статус льготы",
            name='privilege_confirmed',
            read_only=True)

        win.confirm_privilege_button = ext.ExtButton(
            text=u"Просмотр/изменение данных",
            handler='privilegeConfirmation',
            icon_cls=Icons.ACCEPT,
            disabled=True)

        win.form.items.extend([
            win.privilege_owner,
            win.delegate,
            win.privilege_confirmation_status])

        win.buttons.insert(0, win.confirm_privilege_button)

        if context.decl_privilege_id:
            win.privilege_confirmation_url = pack.edit_window_action.get_absolute_url()

            declaration_privilege = action.parent.get_row(
                context.decl_privilege_id)

            try:
                privilegeconfirmationattributes = (
                    declaration_privilege.privilegeconfirmationattributes_set.latest(
                                'id'))
            except PrivilegeConfirmationAttributes.DoesNotExist:
                confirmed = False
                owner = None
                delegate = None
            else:
                confirmed = privilegeconfirmationattributes.confirmed
                owner = privilegeconfirmationattributes.privilege_owner
                delegate = privilegeconfirmationattributes.delegate

            win.privilege_confirmation_status.value = PCS[confirmed]
            win.privilege_owner.value = owner

            if delegate:
                win.delegate.set_value_from_model(delegate)

            win.confirm_privilege_button.disabled = False if owner else True
        else:
            win.privilege_confirmation_url = pack.new_window_action.get_absolute_url()
            win.privilege_confirmation_status.value = PCS[False]

        return win


class PrivilegeEditWindowExtension(object):
    """
    Расширяем окно редактирования льготы
    Добавляем поле "Ведомство" для льготы,
    простовляем текущее значение департамента
    """

    @classmethod
    def build_extension(cls, edit_win):

        edit_win.department = ExtDictSelectField(
            region='north',
            anchor='100%',
            label=u'Ведомство',
            name='department_id',
            display_field='name',
            ask_before_deleting=False,
            hide_edit_trigger=True,
            hide_dict_select_trigger=True,
            hide_trigger=False,
        )
        department_pack = ControllerCache.find_pack(
            'lipetsk_specifics.actions.DepartmentPack')
        edit_win.department.pack = department_pack.__class__
        edit_win.department.editable = False
        edit_win.form.items.append(edit_win.department)

        return edit_win

    @classmethod
    def set_value(cls, edit_win, action, context):
        priv_pack = action.parent
        priv_id = getattr(context, priv_pack.id_param_name)
        # должна быть всегда 1 запись,тк одна льгота принадлежит одному
        # вед-ву, поменять потом на get
        dep_qs = Department.objects.filter(privileges=priv_id)
        if dep_qs:
            edit_win.department.set_value_from_model(dep_qs[0])
        return edit_win


class PrintDeclNotificationWindowExtension(object):
    """
    Окно уведомления для заявлений в статусе Желает изменить ДОУ
    """

    @staticmethod
    def field_by_mode(mode):
        if mode == NOTIFICATION_MODE['WANT_CHANGE_DOU']:
            return (ControllerCache.find_action(
                'lipetsk_specifics.actions.'
                'ReturnNotificationAction').absolute_url(),
                ui_helpers.make_combo_box(
                    data=list(ReturnReasonEnum.get_choices()),
                    label=u'Причины возврата в очередь',
                    name='return_reason',
                    width=380))
        elif mode == NOTIFICATION_MODE['REJECT']:
            return (ControllerCache.find_action(
                'lipetsk_specifics.actions.'
                'RejectNotificationAction').absolute_url(),
                    None)
        else:
            return ControllerCache.find_action(
                'lipetsk_specifics.actions.'
                'PrintNotificationAction').absolute_url(), None

    @classmethod
    def build_extension(cls, win, notification_mode):
        url, addition_field = cls.field_by_mode(notification_mode)
        if addition_field:
            win.form.items.insert(1, addition_field)
        win.form.url = url
        return win


class LogStatusAddWindowExtension(object):
    """
    Добавляем поле "Причина отказа" на форму смены статуса заявки
    """

    @classmethod
    def build_extension(cls, add_win):
        add_win.template_globals = 'ui-js/declaration-reject-reason.js'

        add_win.declaration_status_reject = DSS.REFUSED
        add_win.declaration_reject_reason = ExtDictSelectField(
            label=u'Причина отказа',
            name='reject_reason_id',
            display_field='name',
            ask_before_deleting=False,
            hide_trigger=False,
            hide_dict_select_trigger=True,
            hide_edit_trigger=True,
            anchor='100%',
            allow_blank=False,
            editable=False,
            pack='lipetsk_specifics.actions.RejectReasonPack')
        add_win.form.items.append(add_win.declaration_reject_reason)

        return add_win
